{% extends basetemplate %}

{% block title %}{{ title }}{% endblock %}

{% block content %}

<h1>{{ title }}</h1>

<form method="post" action="">

<table>

{{ form }}

<tr><td></td>
<td><input type="submit" name="submit" value="{{ submit_label }}" /></td></tr>

</table>

</form>

{% endblock %}

