{% extends "template.html" %}

{% block content %}

<h1>Smug Login</h1>

{% if form.errors %}
<p>Your username and/or password were invalid. Please try again.</p>
{% endif %}

<form method="post" action="">
<table>
<tr><td><label for="id_username">Username:</label></td><td>{{ form.username }}</td></tr>
<tr><td><label for="id_password">Password:</label></td><td>{{ form.password }}</td></tr>
</table>

<input type="submit" value="login" />
<input type="hidden" name="next" value="{{ next }}" />
</form>

{% endblock %}
