{% extends "template.html" %}
{% load i18n %}

{% block title %}{% trans 'Page not found' %}{% endblock %}

{% block content %}

<h2>{% trans 'Page not found' %}</h2>

<p>{% trans "We're sorry, but the requested page could not be found." %}</p>

{% endblock %}
